library typed_pipe_config;

import 'package:struct_transfer/struct_transfer.dart';
import "dart:async";
import "dart:typed_data";

IsyumiTransfer getTransfer(
                   Stream<Uint8List> in_data,
                   void out_data(Uint8List data)) => new IsyumiTransfer(new TransferBuilder(in_data,out_data)); 

class IsyumiTransfer extends Transfer{
  
  final ResetStruct reset;
  final UrlStruct url;
  final AppendStruct appendDiv,appendText,appendAnchor,appendTextInput,appendPasswordInput;
  final SetTextStruct setText; 
  final SetHrefStruct setHref;
  final SetStringValueStruct setStringValue;
  
  IsyumiTransfer(TransferBuilder tb):
    super(tb),
    reset = new ResetStruct(tb.getStructBuilder()),
    url = new UrlStruct(tb.getStructBuilder()),
    appendDiv = new AppendStruct(tb.getStructBuilder()),
    appendText = new AppendStruct(tb.getStructBuilder()),
    appendAnchor = new AppendStruct(tb.getStructBuilder()),
    appendTextInput = new AppendStruct(tb.getStructBuilder()),
    appendPasswordInput = new AppendStruct(tb.getStructBuilder()),
    setText = new SetTextStruct(tb.getStructBuilder()),
    setHref = new SetHrefStruct(tb.getStructBuilder()),
    setStringValue = new SetStringValueStruct(tb.getStructBuilder())
  ;
}

class ResetStruct extends Struct{
  ResetStruct(StructBuilder sb):
    super(sb);
}

class UrlStruct extends Struct{
  
  StringParam url;
  
  UrlStruct(StructBuilder sb):
    super(sb),
    url = sb.stringParam();
  
}

class AppendStruct extends Struct{
  final IntListParam id;
  
  AppendStruct(StructBuilder sb):
    super(sb),
    id= sb.intListParam();
  
}

abstract class UIAttrStruct extends Struct{
  final IntListParam id;
  
  UIAttrStruct(StructBuilder sb):
    super(sb),
    id = sb.intListParam();
}

class SetTextStruct extends UIAttrStruct{
  final StringParam text;
  
  SetTextStruct(StructBuilder sb):
    super(sb),
    text = sb.stringParam();
  
}

class SetHrefStruct extends UIAttrStruct{
  final StringParam href;
  
  SetHrefStruct(StructBuilder sb):
    super(sb),
    href = sb.stringParam();
  
}

abstract class SetValueStruct<T> extends UIAttrStruct{
  Param<T> get value;
  SetValueStruct(StructBuilder sb):
    super(sb);
}

class SetStringValueStruct extends SetValueStruct<String>{
  final StringParam value;
  SetStringValueStruct(StructBuilder sb):
    super(sb),
    value = sb.stringParam();
}